// ==================================================================================
// Program..............: ccexampleexport.dpr
// Info.................: clarc example exportservice
// .....................: Standalone example without any special units or classes
// ----------------------------------------------------------------------------------
// Author...............: Dennis Balzuweit
// Company..............: CTO Balzuweit GmbH
// ----------------------------------------------------------------------------------
// Creation.............: 24.10.2014
// Changed..............: 11.03.2019
// ----------------------------------------------------------------------------------
// 24.10.2014 - 1.0.0   : (DB) Erstellung
// 11.03.2019 - 1.0.1   : (DB) Aktualisierung
// ==================================================================================
library ccexampleexport;

{$R *.res}
{$WARN UNIT_PLATFORM OFF}
{$WARN SYMBOL_PLATFORM OFF}

// ==================================================================================
uses
  Windows,
  SysUtils,
  Classes,
  Controls,
  StrUtils,
  Forms,

  ConfigUnit in 'ConfigUnit.pas' {CfgForm};

// ==================================================================================
const
  MyVersion                      = '1.0.0';

const
  ccfCode_ES_Initialize          = 100;
  ccfCode_ES_GetInterfaceVersion = 101;
  ccfCode_ES_GetVersion          = 105;
  ccfCode_ES_SetConfig           = 110;
  ccfCode_ES_SetParameter        = 115;
  ccfCode_ES_SetOEMInfos         = 116;
  ccfCode_ES_GetDocPath          = 120;
  ccfCode_ES_SetDestination      = 130;
  ccfCode_ES_SetExpBuffer        = 160;
  ccfCode_ES_GetExpBuffer        = 170;
  ccfCode_ES_StartDoc            = 200;
  ccfCode_ES_StartStack          = 250;
  ccfCode_ES_AddTitle            = 300;
  ccfCode_ES_AddField            = 400;
  ccfCode_ES_AddTableData        = 450;
  ccfCode_ES_AddBlob             = 500;
  ccfCode_ES_AddBlobName         = 510;
  ccfCode_ES_AddBlobData         = 520;
  ccfCode_ES_AddBlobDocType      = 530;
  ccfCode_ES_AskBlobDelete       = 550;
  ccfCode_ES_EndDoc              = 600;
  ccfCode_ES_EndStack            = 650;
  ccfCode_ES_RollBack            = 700;
  ccfCode_ES_CallCfgDlg          = 800;
  ccfCode_ES_GetConfig           = 850;
  ccfCode_ES_Finalize            = 900;

// ==================================================================================
var
  CForm          : TCfgform;
  ObjList        : TStringlist;
  IdxList        : TStringlist;
  Title          : WideString;
  CfgData        : WideString;
  AppLabel       : WideString;
  ExportFileName : WideString;
  TargetPath     : WideString;

// ==================================================================================
function ccWideExport(fCode : LongInt;
                      fData : PWideChar;
                      fSize : LongInt;
                      Msg   : PWideChar ) : LongInt;

var
  i        : Integer;
  ExpFile  : Textfile;
  Field    : WideString;
  Value    : WideString;
  HStr     : WideString;
  CfgList  : TStringlist;

begin
  Result := 0;

  try
    case fCode of
      // ---------------------------------
      // Init
      // ---------------------------------
      ccfCode_ES_Initialize :
      begin
        AppLabel       := WideString(fData);
        ObjList        := TStringlist.Create;
        IdxList        := TStringlist.Create;

        ExportFileName := '';
      end;
      // ---------------------------------
      // Version
      // ---------------------------------
      ccfCode_ES_GetVersion :
      begin
        StrPCopy(fData,MyVersion);
      end;
      // ---------------------------------
      // Set Configuration
      // ---------------------------------
      ccfCode_ES_SetConfig :
      begin
        CfgList := TStringlist.Create;
        try
          CfgData        := WideString(fData);
          CfgList.Text   := CfgData;

          ExportFileName := CfgList.Values['exportfilename'];
        finally
          CfgList.Free;
        end;
      end;
      // ---------------------------------
      // Blob Path
      // ---------------------------------
      ccfCode_ES_GetDocPath :
      begin
        TargetPath := ExtractFilePath(ExportFileName);
        StrPCopy(fData,TargetPath);
      end;
      // ---------------------------------
      // Set Destination
      // ---------------------------------
      ccfCode_ES_SetDestination :
      begin
        HStr := WideString(fData);
        if Length(HStr) > 0 then ExportFileName := HStr;
      end;
      // ---------------------------------
      // Stack start
      // ---------------------------------
      ccfCode_ES_StartStack:
      begin
      end;
      // ---------------------------------
      // Start Document
      // ---------------------------------
      ccfCode_ES_StartDoc :
      begin
        ObjList.Clear;
        IdxList.Clear;
        Title := '';
      end;
      // ---------------------------------
      // Titel
      // ---------------------------------
      ccfCode_ES_AddTitle :
      begin
        Title := WideString(fData);
      end;
      // ---------------------------------
      // Fields
      // ---------------------------------
      ccfCode_ES_AddField :
      begin
        IdxList.add(WideString(fData));
      end;
      // ---------------------------------
      // Blobs
      // ---------------------------------
      ccfCode_ES_AddBlob :
      begin
        ObjList.add(WideString(fData));
      end;
      // ---------------------------------
      // Delete Blobs?
      // ---------------------------------
      ccfCode_ES_AskBlobDelete :
      begin
        StrPCopy(fData,'false');
      end;
      // ---------------------------------
      // Export (Write file)
      // ---------------------------------
      ccfCode_ES_EndDoc :
      begin
        AssignFile(ExpFile,ExportFileName);
        try
          if FileExists(ExportFileName) = false
            then Rewrite(ExpFile)
            else Append(ExpFile);

          HStr := '';

          for i:=0 to IdxList.count-1 do
          begin
            Field := copy(IdxList.Strings[i],1,pos('=',IdxList.Strings[i])-1);
            Value := copy(IdxList.Strings[i],pos('=',IdxList.Strings[i])+1,length(IdxList.Strings[i]));
            HStr  := HStr + '[' + Field + ']"' + Value + '",';
          end;

          for i:=0 to ObjList.count-1 do
          begin
            HStr := HStr + '[blob]"'+ObjList.Strings[i] + '",';
          end;

          HStr := copy(HStr,1,length(HStr)-1);
          Writeln(ExpFile,HStr);

          StrPCopy(Msg,'export successfully processed');
        finally
          Closefile(ExpFile);
        end;
      end;
      // ---------------------------------
      // Stack End
      // ---------------------------------
      ccfCode_ES_EndStack :
      begin
      end;
      // ---------------------------------
      // Exportservice Finish
      // ---------------------------------
      ccfCode_ES_Finalize :
      begin
        ObjList.Free;
        IdxList.Free;
      end;
      // ---------------------------------
      // Config Dialog
      // ---------------------------------
      ccfCode_ES_CallCfgDlg :
      begin
        CForm := TCfgForm.Create(nil);
        try
          CForm.Edit1.text := ExportFileName;

          if CForm.ShowModal = mrOk then
          begin
            CfgList := TStringList.Create;
            try
              CfgList.Add('exportfilename='+CForm.Edit1.Text);
              CfgData := CfgList.Text;
            finally
              CfgList.Free;
            end;
          end else Result := 1;

          StrPCopy(fData,CfgData);
        finally
          CForm.Free;
        end
      end;
      // ---------------------------------
      // Get Config
      // ---------------------------------
      ccfCode_ES_GetConfig :
      begin
        StrPCopy(fData,CfgData);
      end;

    end;
  except
    on e: Exception do
    begin
      Result := -1;
      StrPCopy(Msg,'export processing exception ('+e.Message+')');
    end;
  end;
end;

// ================================================================
Exports
  ccWideExport;

begin
end.
